var scene = {
"keywords":"",
"publisher":"",
"rights":"",
"description":"",
"language":"",
"title":"generic1",
"image":"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAoAAAAKCAYAAACNMs+9AAAAJ0lEQVR4nGM0NmZ44ObG8J+BEKisZLj//z/Df0KYiaBJUDCqEC8AAImyH5kXO/lqAAAAAElFTkSuQmCC",
"creator":"",
"contributor":"",
"height":"10",
"width":"10",
"relation":"",
"intro_title":"title",
"coverage":"",
"date":"",
"identifier":"",
"intro_detail":"description",
"source":"",
};
var details = [
{
  "minY":"0.0",
  "maxX":"152.0",
  "maxY":"161.0",
  "title":"image title",
  "image":"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAGQAAABkCAYAAABw4pVUAAABJklEQVR4nO3T0QmDABQEwZdgGSlH 7MjexGZCIF0EJGW4HzMVHCz3WNf5bNs8h9sdx/yWbZvnvs/r7jHMXNe8PSNGkBhBYgSJESRGkBhB YgSJESRGkBhBYgSJESRGkBhBYgSJESRGkBhBYgSJESRGkBhBYgSJESRGkBhBYgSJESRGkBhBYgSJ ESRGkBhBYgSJESRGkBhBYgSJESRGkBhBYgSJESRGkBhBYgSJESRGkBhBYgSJESRGkBhBYgSJESRG kBhBYgSJESRGkBhBYgSJESRGkBhBYgSJESRGkBhBYgSJESRGkBhBYgSJESRGkBhBYgSJESRGkBhB YgSJESRGkBhBYgSJESRGkBhBYgSJESRGkBhBYgSJESRmOc+5ZuZ79xBmznPmD1iYDmfiIwEzAAAA AElFTkSuQmCC ",
  "detail":"image description",
  "height":"161",
  "minX":"0.0",
  "width":"152",
  "options":"",
  "y":"0",
  "x":"0",
  "id":"id_image",
},
{
  "style":"",
  "maxY":"10.0",
  "title":"",
  "detail":"",
  "options":"",
  "minX":"0.0",
  "minY":"0.0",
  "x":"0",
  "y":"0",
  "path":"M0.0 0.0C0.0 0.0 10.0 0.0 10.0 0.0C10.0 0.0 10.0 10.0 10.0 10.0C10.0 10.0 0.0 10.0 0.0 10.0C0.0 10.0 0.0 0.0 0.0 0.0 z",
  "id":"id_path",
  "maxX":"10.0",
  "fill":"",
},
{
  "maxX":"50.0",
  "maxY":"50.0",
  "group": [
  {
      "ry":"0",
      "maxX":"10.0",
      "maxY":"10.0",
  "title":"rectangle title",
      "rx":"0",
      "detail":"rectangle desc",
      "height":"10",
      "minX":"0.0",
      "width":"10",
      "options":"",
      "minY":"0.0",
  "path":'M0.0 0.0C0.0 0.0 10.0 0.0 10.0 0.0C10.0 0.0 10.0 10.0 10.0 10.0C10.0 10.0 0.0 10.0 0.0 10.0C0.0 10.0 0.0 0.0 0.0 0.0 z',
      "y":"0",
      "x":"0",
      "id":"id_rect",
  },
  {
      "ry":"0",
      "maxX":"50.0",
      "maxY":"50.0",
  "title":"",
      "rx":"0",
      "detail":"",
      "height":"50",
      "minX":"0.0",
      "width":"50",
      "options":"",
      "minY":"0.0",
  "path":'M0.0 0.0C0.0 0.0 50.0 0.0 50.0 0.0C50.0 0.0 50.0 50.0 50.0 50.0C50.0 50.0 0.0 50.0 0.0 50.0C0.0 50.0 0.0 0.0 0.0 0.0 z',
      "y":"0",
      "x":"0",
      "id":"id_rect2",
  },
  ],
  "title":"rectangle title",
  "detail":"group desc",
  "id":"id_g",
  "minX":"0.0",
  "minY":"0.0",
  "options":"",
},
];
