/*fonts*/
@font-face {
    font-family: 'yanone_regular';
    src: url('../font/yanonekaffeesatz-regular-webfont.eot');
    src: url('../font/yanonekaffeesatz-regular-webfont.eot?#iefix') format('embedded-opentype'),
         url('../font/yanonekaffeesatz-regular-webfont.woff') format('woff'),
         url('../font/yanonekaffeesatz-regular-webfont.svg#yanone_kaffeesatzregular') format('svg');
    font-weight: normal;
    font-style: normal;
}
@font-face {
	font-family: 'yanone_thin';
    src: url('../font/yanonekaffeesatz-thin-webfont.eot');
    src: url('../font/yanonekaffeesatz-thin-webfont.eot?#iefix') format('embedded-opentype'),
         url('../font/yanonekaffeesatz-thin-webfont.woff') format('woff'),
         url('../font/yanonekaffeesatz-thin-webfont.svg#yanone_kaffeesatzthin') format('svg');
    font-weight: normal;
    font-style: normal;
}
/*fin de font*/

/* override Wahid */

html {
    margin:0;
    padding:0;
  /*background: url('../img/background.jpg') no-repeat center center fixed;
  background-size: cover;*/
    background: rgb(130,76,48); /* Old browsers */
    background-image: -moz-linear-gradient(top, rgba(100,57,38,1) 0%, rgba(100,57,38,1) 19%, rgba(130,76,48,1) 19%, rgba(130,76,48,1) 100%); /* FF3.6+ */
    background-image: -webkit-gradient(linear, left top, left bottom, color-stop(0%,rgba(100,57,38,1)), color-stop(19%,rgba(100,57,38,1)), color-stop(19%,rgba(130,76,48,1)), color-stop(100%,rgba(130,76,48,1))); /* Chrome,Safari4+ */
    background-image: -webkit-linear-gradient(top, rgba(100,57,38,1) 0%,rgba(100,57,38,1) 19%,rgba(130,76,48,1) 19%,rgba(130,76,48,1) 100%); /* Chrome10+,Safari5.1+ */
    background-image: -o-linear-gradient(top, rgba(100,57,38,1) 0%,rgba(100,57,38,1) 19%,rgba(130,76,48,1) 19%,rgba(130,76,48,1) 100%); /* Opera 11.10+ */
    background-image: -ms-linear-gradient(top, rgba(100,57,38,1) 0%,rgba(100,57,38,1) 19%,rgba(130,76,48,1) 19%,rgba(130,76,48,1) 100%); /* IE10+ */
    background-image: linear-gradient(to bottom, rgba(100,57,38,1) 0%,rgba(100,57,38,1) 19%,rgba(130,76,48,1) 19%,rgba(130,76,48,1) 100%); /* W3C */
    filter: progid:DXImageTransform.Microsoft.gradient( startColorstr='#643926', endColorstr='#824c30',GradientType=0 ); /* IE6-9 */
    background-repeat: no-repeat;
}
body{
    background: none;
    margin:0;
    border:none;
    padding:0px;
    border-image-width: 0;
}
video {
    width:100%;
}
audio {
    width:100%;
}
a {
    color:#00c6ff;
  font-family: yanone_thin; 
  font-size:1.4em;    
}
a:active, a:focus, a:hover {color:#DDD;}
header {
    width:100%; 
    padding:0; 
    margin:0 auto; 
    /*background:#000;*/
}

header h1 {    
    margin:0;
    top:0;
    left:0px;
    /*background:url(../img/imagedoc-darknoise.png);*/
    display:inline;
    /*max-width:20%;*/
    padding: 0px 1em 0 5px;
    margin-bottom:0; 
    color:white;
    font-family: yanone_thin; 
    font-size: 1.5em;
}

.eyes { 
    background:url(../img/eyes.png) center bottom no-repeat;
    /*margin-left: 10px;*/
    padding:.3em 1em;
}
.eyes:hover {
    background:url(../img/eyes2.png) center bottom  no-repeat;
}
.meta-doc {
    background:url(../img/meta-doc.png) center bottom  no-repeat;
    /*margin-left: 10px;*/
    padding:.3em 1em
}
.meta-doc:hover{
    background:url(../img/meta-doc2.png) center bottom  no-repeat;
}
.infos {
    background:url(../img/infos.png) center bottom no-repeat;
    /*margin-left: 10px;*/
    padding:.3em 1em; 
}
.infos:hover{
    background:url(../img/infos2.png) center bottom  no-repeat;
}
.title-document {display: block;}

@media screen and (max-width: 640px) {
    body {
        font-size:0.7em;
    }
    h1 {
        font-size:20px;
    }
}
#detect, #canvas {
    margin:0;
    padding:0;
}
#container {
    position: relative;
    padding:0px;
    width:100%;
    margin:10px auto;
    border:0px solid #FFF3B2;

    border-image-width: 0;
}

#canvas {
    width:55%;
    margin-top: 50px;
    border:20px solid rgba(255,255,255,0.5);

}
#detect {
    position:absolute;
    /*top:50px;*/
    left:0;
    width:100%;
    height:655px;
}

.response {
    display:none;
}
.unlock {
    width: 80%;
    margin:0px auto;
}

.unlock input[type="submit"]{
   background-image: url(../img/unlock-icon.png) ;
   background-repeat: no-repeat;
   width: 40px;
   height: 40px;
   border:0px;
   background-color:transparent;
}

.unlock input[type="text"]{
   padding: 10px;
   margin:10px;
   background-color: #ffec64;
   border:0px;
   box-shadow: inset 0px 0px 5px #ffab23;
}

.button {
	-moz-box-shadow: 0px 1px 0px 0px #fff6af;
	-webkit-box-shadow: 0px 1px 0px 0px #fff6af;
	box-shadow: 0px 1px 0px 0px #fff6af;
	background:-webkit-gradient(linear, left top, left bottom, color-stop(0.05, #ffec64), color-stop(1, #ffab23));
	background:-moz-linear-gradient(top, #ffec64 5%, #ffab23 100%);
	background:-webkit-linear-gradient(top, #ffec64 5%, #ffab23 100%);
	background:-o-linear-gradient(top, #ffec64 5%, #ffab23 100%);
	background:-ms-linear-gradient(top, #ffec64 5%, #ffab23 100%);
	background:linear-gradient(to bottom, #ffec64 5%, #ffab23 100%);
	filter:progid:DXImageTransform.Microsoft.gradient(startColorstr='#ffec64', endColorstr='#ffab23',GradientType=0);
	background-color:#ffec64;
	-moz-border-radius:6px;
	-webkit-border-radius:6px;
	border-radius:6px;
	border:1px solid #ffaa22;
	display:inline-block;
	cursor:pointer;
	color:#333333;
	font-family:arial;
	font-size:15px;
	font-weight:bold;
	padding:6px 24px;
	text-decoration:none;
	text-shadow:0px 1px 0px #ffee66;
}
.button:hover {
	background:-webkit-gradient(linear, left top, left bottom, color-stop(0.05, #ffab23), color-stop(1, #ffec64));
	background:-moz-linear-gradient(top, #ffab23 5%, #ffec64 100%);
	background:-webkit-linear-gradient(top, #ffab23 5%, #ffec64 100%);
	background:-o-linear-gradient(top, #ffab23 5%, #ffec64 100%);
	background:-ms-linear-gradient(top, #ffab23 5%, #ffec64 100%);
	background:linear-gradient(to bottom, #ffab23 5%, #ffec64 100%);
	filter:progid:DXImageTransform.Microsoft.gradient(startColorstr='#ffab23', endColorstr='#ffec64',GradientType=0);
	background-color:#ffab23;
}
.button:active {
	position:relative;
	top:1px;
}




#content article {
    display:none;
    z-index:100;
    position:absolute;
    border: 5px solid #ddd;
    width: 100%;
    left:-5px;
    margin: 0px auto;
    background:white;
    box-shadow: black 5px 5px 10px;
    overflow-x: hidden;
    overflow-y: auto;
    max-height: 65px;
}
#content article p, h1 {
    margin: 0px;
    width:95%;
    /*margin: 10px;*/
}
#content article audio {
    max-height: 40px;
    margin:10px;
}
#content article h1 {
    font-family: yanone_thin; 
}

.article_close {
    cursor: pointer;
    position:absolute;
    top:0px;
    right:0px;
    width:30px;
}
/* popup */
.overlay {
    position:fixed;
    top:0;
    right:0;
    left:0;
    bottom:0;
    background: rgba(0,0,0,0.6);
}
#popup {
    position:relative;
    border-radius: 20px;
    border: 20px solid #ddd;
    width: 60%;
    margin: 50px auto;
    background:white;
    min-height: 100px;
    box-shadow: black 5px 5px 10px;
}
#popup_text {
    text-align:center;
}
#popup_text img{
    float:left;
    width:20%;
    max-width: 60px;
    margin: 10px;
}
#content #message_success {
    overflow-x:visible;
    overflow-y:visible;
}
#popup_close, #popup_toggle {
    cursor: pointer;
    position:absolute;
    top:-40px;
    right:-40px;
    width:50px;
}
#loader {
    position:fixed;
    display: table;
    text-align:center;
    width:100%;
    height:100%;
    top:0;
    right:0;
    left:0;
    bottom:0;
    background: black;
    z-index:100;
}
#loader div{
    font-family: yanone_thin, Arial;
    font-size: 2em;    
    display:table-cell;
    vertical-align: middle;
    color:white;
}
.videoWrapper16_9 {
    position: relative;
    padding-bottom: 56.25%; 
    padding-top: 5px;
    height: 0;
}
.videoWrapper4_3 {
    position: relative;
    padding-bottom: 75%; 
    padding-top: 5px;
    height: 0;
}
.videoWrapper16_9 iframe {
    position: absolute;
    top: 0;
    left: 0;
    width: 100%;
    height: 100%;
}
.videoWrapper4_3 iframe {
    position: absolute;
    top: 0;
    left: 0;
    width: 100%;
    height: 100%;
}